Package: BiocIntro
Type: Package
Title: Introduction to Bioconductor for High-Throughput Sequence
        Analysis
Version: 0.0.4
Author: Sonali Arora, Marc Carlson, Nathaniel Hayden, Valerie
    Obenchain, Herv\'e Pag\`es, Paul Shannon, Dan Tenenbaum, Martin
    Morgan
Maintainer: Biocore Team c/o BioC user list
        <bioconductor@stat.math.ethz.ch>
License: Artistic-2.0
Description: Introduction to Bioconductor for High-Throughput Sequence
    Analysis introduces users with some R experience to Bioconductor,
    especially working with high-throughput sequence data. Day 1
    develops core R and Bioconductor concepts for working with large
    and complicated data. Participants will become familiar with data
    classes, packages, and scripting and programming concepts that are
    important for common and integrated work flows in
    Bioconductor. Day 2 will put these skills to use for the analysis
    of RNAseq differential expression data, including initial quality
    assessment, pre-processing, differential representation,
    annotation, and visualization.  The course involves a combination
    of presentations and hands-on exercises; participants should come
    prepared with a modern laptop with wireless internet access.
Depends: R (>= 3.1)
Imports: methods, Biostrings, ShortRead, ggplot2, graphics
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, AnnotationHub, BSgenome.Hsapiens.UCSC.hg19,
        BiocParallel, Biostrings, GenomicAlignments, GenomicFeatures,
        GenomicRanges, Gviz, IRanges, PSICQUIC,
        RNAseqData.HNRNPC.bam.chr14, TxDb.Hsapiens.UCSC.hg19.knownGene,
        VariantAnnotation, biomaRt, knitr, org.Hs.eg.db, parallel,
        rtracklayer
Packaged: 2014-03-02 19:50:25 UTC; mtmorgan
