import(methods)

importClassesFrom(Biostrings, DNAStringSet)

importClassesFrom(ShortRead, QualityScore)

importMethodsFrom(Biostrings, encoding)

importMethodsFrom(ShortRead, alphabetByCycle, FastqStreamer, trimTailw,
                  writeFastq, yield, quality)

importFrom(Biostrings, DNAStringSet)

importFrom(ggplot2, ggplot, aes, geom_boxplot, scale_x_discrete)

importFrom(graphics, legend, matplot)

export(plotByCycle, trimFastq)
