% \VignetteEngine{knitr::knitr}
% \VignetteIndexEntry{01. Introduction}

\documentclass{article}

<<style, eval=TRUE, echo=FALSE, results="asis">>=
BiocStyle::latex()
@ 

\title{Introduction to \Bioconductor{} for High-Throughput Sequence Analysis}
\author{Martin T.\ Morgan (\url{mtmorgan@fhcrc.org})\\
    Sonali Arora (\url{sarora@fhcrc.org})}
\date{27-28 February 2014}

\begin{document}

\maketitle

Introduction to Bioconductor for Sequence Analysis introduces users
with some R experience to Bioconductor, especially working with
high-throughput sequence data. Day 1 develops core R and Bioconductor
concepts for working with large and complicated data. Participants
will become familiar with data classes, packages, and scripting and
programming concepts that are important for common and integrated work
flows in Bioconductor. Day 2 will put these skills to use for the
analysis of RNAseq differential expression data, including initial
quality assessment, pre-processing, differential representation,
annotation, and visualization.  The course involves a combination of
presentations and hands-on exercises; participants should come
prepared with a modern laptop with wireless internet access.

A preliminary schedule is in Table~\ref{tab:schedule}.

\begin{table}
  \centering
  \caption{Preliminary schedule, Introduction to Bioconductor for 
    Sequence Analysis}
  \label{tab:schedule}
  \begin{tabular}{llp{.6\textwidth}}
    \hline\noalign{\smallskip}
    \multicolumn{3}{l}{Day 1: \R{} and \Bioconductor{}} \\    
    & 9:00 - 12:30 &  
      Working with \R{} classes, help, programming concepts, and packages.
      Introduction to high-throughput sequence work flows. \\
    & 1:30 - 5:00 & 
      \Bioconductor{} classes and packages for sequence analysis.\\
    \multicolumn{3}{l}{Day 2: RNA-seq Annotation and Visualization} \\
    & 9:00 - 12:30 &  
      RNA-seq work flows. Quality assessement and pre-processing. 
      Differential expression. \\
    & 1:30 - 5:00 & 
      RNA-seq down-stream and alternative analysis. 
      Annotation and visualization.\\
    \hline
  \end{tabular}
\end{table}

\end{document}
