\name{BiocIntro-package}
\Rdversion{1.1}
\alias{BiocIntro-package}
\alias{BiocIntro}
\docType{package}

\title{Introduction to Bioconductor for High-Throughput Sequence
  Analysis}

\description{

  This package accompanies the course `Introduction to Bioconductor for
  High-Throughput Sequence Analysis'. The course introduces users with
  some R experience to Bioconductor, especially working with
  high-throughput sequence data. Day 1 develops core R and Bioconductor
  concepts for working with large and complicated data. Participants
  will become familiar with data classes, packages, and scripting and
  programming concepts that are important for common and integrated work
  flows in Bioconductor. Day 2 will put these skills to use for the
  analysis of RNAseq differential expression data, including initial
  quality assessment, pre-processing, differential representation,
  annotation, and visualization.  The course involves a combination of
  presentations and hands-on exercises; participants should come
  prepared with a modern laptop with wireless internet access.

}

\author{

  Author: Sonali Arora, Marc Carlson, Nathaniel Hayden Valerie
  Obenchain, Herv\'e Pag\`es, Paul Shannon, Dan Tenenbaum, Martin Morgan.

  Maintainer: Biocore Team c/o BioC user list
  <bioconductor@stat.math.ethz.ch>

}

\keyword{ package }

\examples{
packageDescription("BiocIntro")
vignette(package="BiocIntro")    ## available vignettes
}
